use eyre::{eyre, Result};
use serde::{Deserialize, Serialize};

use crate::api::models::lights::{Light, On};
use crate::api::models::Response;
use crate::config::Config;
use crate::{api, ToggleState};

pub async fn all(config: Config) -> Result<Vec<Light>> {
    let request_url = format!("https://{}/clip/v2/resource/light", config.hostname);
    let client = api::get_client(&config)?;

    match client.get(request_url).send().await {
        Ok(response) => Ok(response.json::<Response<Light>>().await?.data),
        Err(e) => {
            if let Some(status_code) = e.status() {
                Err(eyre!(
                    "Failed to get all lights, status code was {:?}",
                    status_code
                ))
            } else {
                Err(eyre!("Failed to get all lights"))
            }
        }
    }
}

#[derive(Serialize, Deserialize)]
pub struct LightStatePut {
    on: On,
}

pub async fn toggle(config: Config, id: String, state: ToggleState) -> Result<()> {
    let request_url = format!("https://{}/clip/v2/resource/light/{}", config.hostname, id);
    let client = api::get_client(&config)?;

    let target_state: LightStatePut = LightStatePut {
        on: On {
            on: match state {
                ToggleState::On => true,
                ToggleState::Off => false,
                ToggleState::Swap => match all(config).await?.into_iter().find(|x| x.id == id) {
                    Some(light) => !light.on.on,
                    None => true,
                },
            },
        },
    };

    client.put(request_url).json(&target_state).send().await?;

    Ok(())
}

// pub async fn get_light() -> Result<Light> {
//     let request_url = format!("https://{}/clip/v2/resource/light/{}", api::HUB_IP, id);
//     let client = api::get_client().unwrap();
//
//     match client.get(request_url).send().await {
//         Ok(response) => {
//             let deserialized = response.json::<Response<Light>>().await?;
//             Ok(deserialized.data.first()?)
//         },
//         Err(e) => {
//             if let Some(status_code) = e.status(){
//                 Err(eyre!("Failed to get all lights, status code was {:?}", status_code))
//             } else {
//                 Err(eyre!("Failed to get all lights"))
//             }
//         }
//     }
// }
