use clap::{Parser, Subcommand};
use std::str::FromStr;

#[derive(Parser)]
#[clap(author, version, about, long_about = None)]
#[clap(propagate_version = true)]
pub struct Args {
    #[clap(subcommand)]
    pub command: Commands,
}

#[derive(Subcommand)]
pub enum Commands {
    /// List all lights
    List,
    /// Inspect a light
    Inspect { id: String },
    /// Toggle a light
    Toggle { id: String, state: Option<String> },
    /// Get a token
    Auth,
}

#[derive(Debug, Copy, Clone, PartialEq, Eq, PartialOrd, Ord)]
pub enum ToggleState {
    On,
    Off,
    Swap,
}

impl FromStr for ToggleState {
    type Err = ();

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        let res = match s {
            "on" => ToggleState::On,
            "off" => ToggleState::Off,
            "swap" => ToggleState::Swap,
            _ => ToggleState::Swap,
        };

        Ok(res)
    }
}
