use eyre::Result;
use serde::Deserialize;
use std::fs;
use std::path::Path;

#[derive(Deserialize)]
pub struct ConfigFile {
    pub token: Option<String>,
    pub host: Option<Host>,
}

#[derive(Deserialize)]
pub struct Host {
    pub hostname: String,
}

pub async fn read() -> Result<Option<ConfigFile>> {
    let home_dir = match dirs::home_dir() {
        Some(h) => h,
        None => return Ok(None),
    };

    let config_file_path = home_dir.join(Path::new(".hue-control.toml"));

    let content = match fs::read_to_string(config_file_path) {
        Ok(c) => c,
        Err(_) => return Ok(None),
    };

    let config: ConfigFile = toml::from_str(&content)?;

    Ok(Some(config))
}
