mod api;
mod args;
mod commands;
mod config;

use crate::args::ToggleState;
use clap::Parser;
use eyre::{eyre, Result};

#[tokio::main]
async fn main() -> Result<()> {
    let args = args::Args::parse();
    let config = config::resolve().await?;

    let res = match &args.command {
        args::Commands::Toggle { id, state } => {
            commands::toggle::exec(config, id, state).await
        }
        args::Commands::List => {
            commands::list::exec(config).await
        }
        args::Commands::Inspect { id } => {
            commands::inspect::exec(config, id).await
        }
        args::Commands::Auth => {
            Err(eyre!("Not yet implemented"))
        }
    };

    if let Err(e) = res {
        println!("⛔️ {}", e.to_string());
        std::process::exit(1);
    }

    Ok(())
}
